unit uoddface_simple;

interface

uses
  Classes, SysUtils,
  Math,
  urosx,
  xmatdefs,
  uauxiliares,
  process,
  uConstantesSimSEE,
  uDataSetGenerico;

const
  // constantes modificables por el usuario
  verboso = 0; // poner a 0 para evitar salidas que no sean de error.
  ntares_por_vez = 10; // cantidad de tareas que intenta lazar cada vez.
  pasar_idEjecutor = True;
  // poner a TRUE para pasarle a la aplicaci�n un idEjecutor �nico.

  pausa_segs = 1;
  // pausa en segundo entre lanzamientos. Para darle tiempo a cada demonio a copiar los datos comunes a su /tmp sin interferirse.

  qstat_cmd_ = 'schtasks';

  qsub_cmd_ = '/opt/torque/bin/qsub'; // '/opt/pbs/bin/qsub';
  showq_cmd_ = '/opt/maui/bin/showq';
  crontab_cmd_ = '/usr/bin/crontab';



type

  { TConfigRec }

  TConfigRec = class
    IpFija: string;
    Host: string;
    Puerto: string;
    uri: string;
    usuario_http: string;
    clave_http: string;
    inodo: integer;
    carpeta_tmp: string;
    maxnucleos: integer;
    maxtareas: integer;
    maxhoras: integer;
    constructor CreateFromArchi( archi: string );
    procedure WriteToArchi( archi: string );
  end;


  { TCola }
  TCola = class
    nombre: string;
    nnucleos: integer;
    maxtareas: integer;
    maxhoras: integer;
    constructor Create(nombre: string; nnucleos, maxhoras, maxtareas: integer);
  end;

  { TRBT_OddFace_Simple }

  TRBT_OddFace_Simple = class
    usuario: string;
    home: string;
    yo: string;
    offset_ejecutor: integer;
    aplicacion: string;
    orden: string;
    colas: TList;
    ncolas: integer;
    s: ansistring;
    sql: string;
    db: TDBrosxCon;
    ds: TResultadoQuery;

    cfg: TConfigRec;

    constructor Create;

    // ejecuta el comando y retorna la última linea. En mls retorna un stringlist con todas las lineas de salida.
    function exec(apl: string; params: array of string;
      var mls: TStringList; flgWaitOnExit: boolean = true ): string; overload;
    function exec(comando: string; params: array of string;
      flgWaitOnExit: boolean=true): string; overload;

    // Recibe un array de constantes y retorna un TDAofString con las mismas
    function daofs(a: array of string): TDAofString;

    // writeln si vervoso > 1
    procedure wrln(s: string);

    // writeln independientemente del valor de vervoso.
    procedure wrln_err(s: string);

    // verifica si el programa oddface_repetidoro se está ejecutando y
    // si no lo está lo ejecuta. Este procedimiento se utiliza para mantener
    // en ejecución el repetidor.
    procedure test_repetidor;

(*
  nexval( $key, $reserva = 1) , retorna un identificador único global
  el parámetro $reserva indica la cantidad de identificadores consecutivos
  reservados (es decir asigandos y que no volveran ser asignados.
 *)
    function nextval(key: string; reserva: integer = 1): integer;


    procedure remover_crontab;
    procedure instalar_crontab;

    // retorna la cantidad de trabajos esperando para ser despachados
    function getRunning: integer;

    // emulador de substr de php
    function substr(const s: string; k_base_0: integer; cnt: integer = -1): string;

    procedure Ayuda;
    procedure lanzar(ntareas: integer; xcola: TCola);
    procedure runLanzarParar(xcola: TCola);
    procedure parar(ntareas: integer);

    // retorna TRUE si la orden es válida.
    function ProcesarOrden: boolean;

    procedure orden_run;
    procedure order_parar_tareas;

    procedure dispatch_cmd;


  private
    procedure orden_lst;
    procedure orden_cancelartrabajos(username, xcola, wk_nidProblema: string);
  end;


implementation

type
  TDAofInteger = array of integer;


function MedirCols(r: string; ColsCap: array of string): TDAOfInteger;
var
  k, i: integer;
  res: TDAOfInteger;
begin
  setlength(res, length(ColsCap));
  for k := 0 to high(ColsCap) do
  begin
    i := pos(ColsCap[k], r);
    if i = 0 then
      raise Exception.Create('ColsCap no encontrada: ' + ColsCap[k]);
    res[k] := i + length(ColsCap[k]);
  end;
  Result := res;
end;

function getColVal(r: string; icols: TDAofInteger; kCol: integer): string;
var
  res: string;
begin
  if kCol = 0 then
    res := copy(r, 1, icols[0] - 1)
  else
    res := copy(r, icols[kCol - 1], icols[kCol] - icols[kCol - 1]);
  Result := trim(res);
end;

{ TConfigRec }

constructor TConfigRec.CreateFromArchi(archi: string);

var
  f: TFileStream;

function rp: string;
var
  r: string;
  i: integer;
begin
  r:= f.ReadAnsiString;
  i:= pos( '//', r );
  if i > 0 then
    delete( r, i, length( r ) -i+1 );
  result:= trim( r );
end;


begin
  f:= TFileStream.Create( archi, fmOpenRead + fmShareDenyWrite );
  IpFija:= rp;
  Host:= rp;
  Puerto:= rp;
  uri:= rp;
  usuario_http:=rp;
  clave_http:= rp;
  inodo:= strToInt( rp );
  carpeta_tmp:= rp;
  maxnucleos:= StrToInt( rp );
  maxtareas:= StrToInt( rp );
  maxhoras:= StrToInt( rp );
  f.Free;
end;



procedure TConfigRec.WriteToArchi(archi: string);

var f: TFileStream;

begin
  f:= TFileStream.Create( archi, fmCreate  );
  f.WriteAnsiString( IpFija );
  f.WriteAnsiString( Host );
  f.WriteAnsiString( Puerto );
  f.WriteAnsiString( uri );
  f.WriteAnsiString( usuario_http );
  f.WriteAnsiString( clave_http );
  f.WriteAnsiString( IntToStr( inodo ) );
  f.WriteAnsiString( carpeta_tmp );
  f.WriteAnsiString( IntToStr( maxnucleos ));
  f.WriteAnsiString( IntToStr( maxtareas ));
  f.WriteAnsiString( IntToStr( maxhoras ));
  f.Free;
end;

{ TCola }
constructor TCola.Create(nombre: string; nnucleos, maxhoras, maxtareas: integer);
begin
  self.nombre := nombre;
  self.nnucleos := nnucleos;
  self.maxhoras := maxhoras;
  self.maxtareas := maxtareas;
end;


constructor TRBT_OddFace_Simple.Create;
begin

  usuario := GetEnvironmentVariable('USER');
  home := GetEnvironmentVariable('HOME');
  yo := ParamStr(0);

  cfg:= TConfigRec.CreateFromArchi( getDir_Bin + 'oddface_simple.cfg' );

  offset_ejecutor := 0;
  if (usuario = 'simsee') then
    offset_ejecutor := 1000; // para diferenciarlo del usuario rchaer

  aplicacion := '';

  colas := TList.Create;
  colas.add(TCola.Create('normal', 560, 12, 60));  // maxhoras = 10 * 24

  ncolas := colas.Count;

  if (ParamCount >= 1) then
    orden := ParamStr(1)
  else
    orden := '?';

  sql := 'SELECT now()';
  db := TDBrosxCon.Create('179.27.96.77', '', '80', '/oddface/rosx.php');


  ds := DB.sql_query(sql);
  if (ds = nil) or (ds.nrows = 0) then
  begin
    ds.Free;
    DB.Free;
    writeln('Error de conectividad con la DB remota ');
    exit;
  end;
  //  writeln('La hora del DB remoto es: ', ds.fields[0].AsString);
end;

function TRBT_OddFace_Simple.exec(apl: string; params: array of string;
  var mls: TStringList; flgWaitOnExit: boolean = true ): string;
var
  s: ansistring;
  res: boolean;
begin

  if flgWaitOnExit then
    res:= RunCommandIndir('', apl, params, s, [poWaitOnExit])
  else
    res:= RunCommandIndir('', apl, params, s, []);

  if res then
  begin
    mls := TStringList.Create;
    mls.LineBreak := #10;
    mls.Text := s;
    if mls.Count > 0 then
      Result := mls[mls.Count - 1]
    else
      Result := '';
  end
  else
    raise Exception.Create('Error TRBT_OddFace_Clustger.exec, comando: ' + apl);

end;


function TRBT_OddFace_Simple.exec(comando: string; params: array of string; flgWaitOnExit: boolean = true ): string;
var
  mls: TStringList;
begin
  Result := exec(comando, params, mls, flgWaitOnExit );
  mls.Free;
end;

function TRBT_OddFace_Simple.daofs(a: array of string): TDAofString;
var
  res: TDAOfString;
  k: integer;
begin
  setlength(res, length(a));
  for k := 0 to high(res) do
    res[k] := a[k];
  Result := res;
end;


procedure TRBT_OddFace_Simple.wrln(s: string);
begin
  if (verboso > 0) then
    writeln(s);
end;


procedure TRBT_OddFace_Simple.wrln_err(s: string);
begin
  writeln(s);
end;


(*
  nexval( $key, $reserva = 1) , retorna un identificador �nico global
  el par�metro $reserva indica la cantidad de identificadores consecutivos
  reservados (es decir asigandos y que no volveran ser asignados.
 *)
function TRBT_OddFace_Simple.nextval(key: string; reserva: integer = 1): integer;
var
  f: textfile;
  res, resx: integer;
  archi: string;
begin
  archi := home + '/nextval_' + key;
  if (fileexists(archi)) then
  begin
    assignfile(f, archi);
    reset(f);
    readln(f, res);
    closefile(f);
  end
  else
    res := 1;

  resx := res + reserva;

  assignfile(f, archi);
  rewrite(f);
  writeln(f, resx);
  closefile(f);
  Result := res + offset_ejecutor;
end;




procedure TRBT_OddFace_Simple.test_repetidor;
var
  s: ansistring;
  m: TStringList;
begin
    (*
UID        PID  PPID  C STIME TTY          TIME CMD
rchaer   24106     1  0 13:37 ?        00:00:00 /home/rchaer/simsee/fpcbin/oddface_repetidor
  *)
  //   exec('ps -f -C oddface_repetidor', $m);
  RunCommand('/bin/bash', ['-c', 'ps -f -C oddface_repetidor'], s);

  m := TStringList.Create;
  m.Text := s; // se supone que esto la parsea
  if (m.Count <= 1) then
  begin
    writeln('relanzando repetidor...');
    LanzarApl(home + '/SimSEE/bin/oddface_repetidor', []);
  end;
end;



procedure TRBT_OddFace_Simple.remover_crontab;
var
  m: TStringList;
  ul: string;
begin
  ul := exec( crontab_cmd_, ['/delete', '/tn', '"rbt_oddFece_simple"'], m);
  m.Free;
end;



procedure TRBT_OddFace_Simple.instalar_crontab;

var
  ul: string;
  m: TStringList;
begin
  ul := exec(crontab_cmd_, [
   '/create', '/sc', 'minute', '/mo', '3', '/tn', '"rbt_oddFece_simple"',
   '/tr', getDir_Bin+'rbt_oddface_simple.exe'], m);
  m.Free;
end;


function TRBT_OddFace_Simple.substr(const s: string; k_base_0: integer;
  cnt: integer): string;
begin
  if cnt > 0 then
    Result := copy(s, k_base_0 + 1, cnt)
  else
    Result := copy(s, k_base_0 + 1, length(s) - k_base_0);
end;

procedure TRBT_OddFace_Simple.Ayuda;
begin
  if orden <> '' then
    writeln('orden desconocida: ' + orden);
  writeln('La sintaxis es ./rbt_OddFace_Simple orden [par2]');
  writeln('  Las ordenes conocidas son: ');
  writeln('    "run" que testea el estado del cluster y lanza o para aplicaciones');
  writeln('    "parar_tareas n" que manda parar "n" tareas. ');
  writeln('    "parar_crontab" elimina el crontab del usuario. ');
  writeln('    "instalar_crontab" instala el crontab del usuario. ');
  writeln('    "parar_todo" para las tareas y borra el crontab ');
  writeln('    "lst" resume cantidad de trabajos en ejecución. ');
  writeln('    "cancelar_trabajos [nidProblema]" cancela todos los trabjos de usuario. ');
end;

procedure TRBT_OddFace_Simple.orden_lst;
var
  nEsperandoTodos, nEsperandoPropios, nTotalTodos, nTotalPropios: integer;
  kCola: integer;
  xcola: TCola;
begin
  for kCola := 0 to colas.Count - 1 do
  begin
    xcola := colas[kCola];
    nTotalTodos:= getRunning;
    writeln('Cola: ', xcola.nombre, ' ============================');
    writeln('TotalTodos: ', nTotalTodos);
  end;
end;




procedure TRBT_OddFace_Simple.orden_cancelartrabajos(
  username, xcola, wk_nidProblema: string);
var
  s: string;
  m: TStringList;
  n, k: integer;
  cnt: integer;
  job_id, Name, usuario, time_use, estado, colax: string;
  icols: array of integer;
  xcola9: string;

begin
  s := exec('tasklist', ['/fi', '"imagename eq oddface_pig.exe"'], m);
  n := m.Count-2;
  for k := 1 to n - 1 do
  begin
    //      wrln( "k: $k, ln: ".$m[$k].":" );
    s := m[k];
    name:= getNextPalSep( s, ' ');
    job_id := getNextPalSep( s, ' ' );
    exec( 'taskkill', [ '/F', '/PID', job_id]  );
    Inc(cnt);
  end;
  Writeln('se cancelaron ', cnt, ' trabajos' );
end;


procedure TRBT_OddFace_Simple.parar(ntareas: integer);
var
  sql: string;
begin
  sql := 'UPDATE ofe_cnt_terminar SET cnt = ' + IntToStr(ntareas) + ' WHERE 1';
  DB.Exec(sql);
end;



procedure TRBT_OddFace_Simple.runLanzarParar(xcola: TCola);
var
  nRunning: integer;
  margen: integer;
begin
  nRunning := getRunning;
  margen  := xcola.maxtareas - nRunning;
  if margen > 0 then
      lanzar( min(margen, ntares_por_vez), xcola);
end;



procedure TRBT_OddFace_Simple.lanzar(ntareas: integer; xcola: TCola);
var
  kbase, k, selk, nrows: integer;
  rec: TDataRecord;
  id_problema, apl_problema: string;
  aplicacion: string;
  idEjecutor: integer;
  idTrabajo: string;
  archi: string;
  fp: textfile;
  r, ul: string;
begin
  kbase := nextval('ejecutores', ntareas);
  nrows := ds.nrows;
  for  k := 0 to ntareas - 1 do
  begin
    selk := random(nrows);
    ds.go(selk);
    rec := ds.getFilaActiva;
    id_problema := trim(rec.fields[0].AsString);
    apl_problema := trim(rec.fields[1].AsString);
    aplicacion := getDir_Bin + apl_problema;

    wrln('nrows: ' + IntToStr(nrows) + ' , selk: ' + IntToStr(selk) +
      ' , apl_problema: ' + apl_problema + ' , id_problema: ' + id_problema);

    if ((apl_problema <> '') and fileexists(aplicacion)) then
    begin
      idEjecutor := kbase + k;

      if (pasar_idEjecutor) then
      begin
        r := aplicacion + ' ' + IntToStr(idEjecutor) + ' ' + id_problema;
        ul := exec( aplicacion, [ IntToStr(idEjecutor), id_problema ]);
      end
      else
      begin
        r := aplicacion + ' ' + id_problema;
        ul := exec( aplicacion, [ id_problema ]);
      end;

      wrln('idEjecutor: ' + IntToStr(idEjecutor) + ', exec: ' + ul );
      sleep(pausa_segs * 1000);
    end;
  end;
end;


(*
  getRunning() retorna la cantidad de tareas ejecutando
 *)
function TRBT_OddFace_Simple.getRunning: integer;
var
  s: string;
  m: TStringList;
  n: integer;
begin
  s := exec('tasklist', ['/fi', '"imagename eq putty.exe"'], m);
  n := m.Count;
  m.Free;
  result := n-2;
end;




procedure TRBT_OddFace_Simple.orden_run;
var
  kcola: integer;
  sql: string;
  cluster_fing: string;
begin
  sql := 'SELECT valor FROM ofe_parametros WHERE nombre=''cluster_fing'' LIMIT 1 ';
  ds := DB.sql_query(sql);
  if (ds.nrows = 0) then
  begin
    writeln('No parameters found for Cluster_fing');
    exit;
  end;

  cluster_fing := trim(ds.GetField(0).AsString);

  if (cluster_fing <> 'ON') then
  begin
    if (cluster_fing = 'OFF') then
    begin
      writeln('Cluster_fing is OFF');
      exit;
    end;
    if (cluster_fing = 'STOP') then
    begin
      sql := 'UPDATE ofe_cnt_terminar SET cnt = 999999 WHERE 1';
      DB.exec(sql);
      sql :=
        'UPDATE ofe_parametros SET valor = ''OFF'' WHERE nombre= ''cluster_fing'' LIMIT 1';
      DB.exec(sql);
      writeln('Cluster_fing is STOPPED');
      exit;
    end;

    if (cluster_fing = 'START') then
    begin
      sql := 'UPDATE ofe_cnt_terminar SET cnt = 0 WHERE 1';
      DB.exec(sql);
      sql :=
        'UPDATE ofe_parametros SET valor = ''ON'' WHERE nombre= ''cluster_fing'' LIMIT 1';
      DB.exec(sql);
    end;
  end;

    (*     * **** rch@20120930
      le agrego consulta a la base de datos de problemas oddface
     * ** *)
  sql := 'SELECT ofe_problemas.nid, ofe_tipos_problema.apl ' +
    'FROM ofe_problemas LEFT JOIN ofe_tipos_problema ON ofe_problemas.tipo = ofe_tipos_problema.nid '
    + ' JOIN ofe_grupo_ucluster ON ofe_problemas.grupo = ofe_grupo_ucluster.grupo ' +
    ' WHERE ofe_grupo_ucluster.ucluster = ''' + Usuario + ''' AND ofe_problemas.estado > 0 ';


  ds := DB.sql_query(sql);

  if (ds.nrows = 0) then
  begin
    wrln('No problems to solve.');
    exit;
  end;

  for kcola := 0 to ncolas - 1 do
  begin
    runLanzarParar(colas[kcola]);
  end;
  exit;
end;


procedure TRBT_OddFace_Simple.order_parar_tareas;
var
  nTareas: integer;
begin
  if (ParamCount > 1) then
    nTareas := StrToInt(ParamStr(2))
  else
    nTareas := 10000; //para todas

  parar(nTareas);
  exit;
end;

procedure TRBT_OddFace_Simple.dispatch_cmd;
var
  NombreCola, nHilos, email: string;
  k: integer;
  id_problema, apl_problema: string;
  aplicacion: string;
  idEjecutor: integer;
  idTrabajo: string;
  archi: string;
  fp: textfile;
  r, ul: string;
  xCola: TCola;
  buscando: boolean;
begin
  id_problema := '0';
  email := ParamStr(1);
  nombreCola := ParamStr(2);
  nHilos := ParamStr(3);
  if StrToInt(nHilos) > 40 then
  begin
    writeln('El máximo de nHilos es 40 en el CLusterUY');
    exit;
  end;
  apl_Problema := ParamStr(4);

  buscando := True;
  for k := 0 to colas.Count - 1 do
  begin
    xCola := colas[k];
    if xCola.nombre = nombreCola then
    begin
      buscando := False;
      break;
    end;
  end;

  if buscando then
  begin
    writeln('Nombre de cola inválido: ', nombreCola);
    exit;
  end;

  idEjecutor := nextval('ejecutores', 1);

  aplicacion := home + '/SimSEE/bin/' + apl_Problema;
  if not fileexists(aplicacion) then
  begin
    writeln('No existe la aplciación: ' + aplicacion);
    exit;
  end;

  idTrabajo := 'wk_' + id_problema + '_' + IntToStr(idEjecutor);
  archi := home + '/' + idTrabajo + '.sh';
  assignfile(fp, archi);
  rewrite(fp);
  writeln(fp, '#!/bin/bash');
  writeln(fp, '#SBATCH --job-name=' + idTrabajo);
  writeln(fp, '#SBATCH --ntasks=1');
  writeln(fp, '#SBATCH --cpus-per-task=' + nHilos);
  //      writeln( fp,  '#SBATCH --mem-per-cpu=40' );
  writeln(fp, '#SBATCH --mem=' + IntToStr(2048 * StrToInt(nHilos)));
  //      writeln( fp,  '#SBATCH --mem=2048' );
  writeln(fp, '#SBATCH --tmp=20G');

  writeln(fp, '#SBATCH --time=' + IntToStr(xcola.maxhoras) + ':00:00');
  //     writeln( fp,  '#SBATCH --time=02:00:00' );
  writeln(fp, '#SBATCH --partition=' + xcola.nombre);

  if xcola.nombre = 'besteffort' then
    writeln(fp, '#SBATCH --qos=besteffort');

  writeln(fp, '#SBATCH --mail-type=FAIL');
  //      writeln( fp,  '#SBATCH --mail-type=ALL' );
  writeln(fp, '#SBATCH --mail-user=rchaer@fing.edu.uy');
  writeln(fp, '#SBATCH --output=/dev/null');
  //     writeln( fp,  '#SBATCH --error=/dev/null');
  writeln(fp, '#SBATCH --open-mode=truncate');

  writeln(fp, 'source /etc/profile.d/modules.sh');

  //   cd ~/miaplicacion
  //  ./mibinario arg1 arg2
  if (pasar_idEjecutor) then
    r := aplicacion + ' ' + IntToStr(idEjecutor) + ' ' + id_problema
  else
    r := aplicacion + ' ' + id_problema;
  writeln(fp, r);

  closefile(fp);

      {$IFNDEF EMULADOR_CLUSTER_UY}
  ul := exec('sbatch', [archi]);
  wrln('idEjecutor: ' + IntToStr(idEjecutor) + ', exec: ' + ul +
    ', cola: ' + xcola.nombre);
      {$ENDIF}
end;



procedure GenerarEjecutorOptSimRes();
begin
  (*
  ejecutor=35

nhilos=48
ntareas={$nhilos}


### configuracion de directorios
homedir={$HOME}
bindir={$homedir}/simsee/fpcbin
corridas={$homedir}/simsee/corridas

tmprundir=/scratch/${USER}
mkdir -p {$tmprundir}/corridas

tmpdirsala=${tmprundir}/corridas/{$ejecutor}
mkdir -p tmpdirsala

archi_defs=${tmprundir}/${ejecutor}/${NombreSala}_${escenario}_${planillaSimres3}.txt

rm -rf ${tmprundir}/${ejecutor}/*

cd {$corridas}
cp {$corridas}/${NombreSala}.zip {$tmpdirsala}
unzip -o ${NombreSala}.zip -d ${NombreSala}
rm -f ${NombreSala}.zip

cd {$tmpdirsala}/${NombreSala}
{$bindir}/cmdopt sala={$tmpdirsala}/${NombreSala}/${NombreSala}.ese ejecutor=${ejecutor} escenario=${escenario} semilla=31 nhilos={$nhilos} ntareas={$ntareas}

cd {$tmpdirsala}/${NombreSala}
{$bindir}/cmdsim sala={$tmpdirsala}/${NombreSala}/${NombreSala}.ese ejecutor=${ejecutor} escenario=${escenario} semilla=31 nhilos={$nhilos} ntareas={$ntareas}

cd ${tmprundir}/${ejecutor}
{$bindir}/cmdsimres3 ${archi_defs} resultados ${ejecutor}

cd ${tmprundir}/${ejecutor}
zip -r ${NombreSala}_${escenario}_resultados.zip *

cp ${NombreSala}_${escenario}_resultados.zip {$corridas}/

rm -f ${NombreSala}_${escenario}_resultados.zip
rm -rf ${tmprundir}/${ejecutor}/*
rm -rf ${tmpdirsala}/*
*)
end;

function TRBT_OddFace_Simple.ProcesarOrden: boolean;
var
  aCola: TCola;
  kCola: integer;
  wk_NidProblema: string;

begin
  if (orden = 'cancelar_trabajos') then
  begin
    if ParamCount > 1 then
      wk_NidProblema := 'wk_' + ParamStr(2)
    else
      wk_NidProblema := '';

    for kCola := 0 to Colas.Count - 1 do
    begin
      aCola := Colas[kCola];
      orden_cancelartrabajos(usuario, aCola.nombre, wk_NidProblema);
    end;

    Result := True;
    exit;
  end;


  if (orden = 'lst') then
  begin
    orden_lst;
    Result := True;
    exit;
  end;


  if (orden = 'run') then
  begin
    //    test_repetidor();
    orden_run;
    Result := True;
    exit;
  end;

  if (orden = 'parar_tareas') then
  begin
    order_parar_tareas;
    Result := True;
    exit;
  end;

  if (orden = 'parar_crontab') then
  begin
    remover_crontab();
    Result := True;
    exit;
  end;

  if (orden = 'parar_todo') then
  begin
    parar(199000);
    remover_crontab();
    Result := True;
    exit;
  end;

  if (orden = 'instalar_crontab') then
  begin
    parar(0);
    instalar_crontab();
    Result := True;
    exit;
  end;

  Result := False;
end;


end.
